@@ -157,13 +157,14 @@ const char * const config_t::s_videoDriverNamesFB [ FB_DRIVER_count+1 ] = {
 };
 
 const char * const config_t::s_frontends[ FRONTEND_count+1 ] = {
-  "sxfe", "fbfe", "none",
+  "sxfe", "fbfe", "wlfe", "none",
   NULL
 };
 
 const char * const config_t::s_frontendNames[ FRONTEND_count+1 ] = {
   "X11 (sxfe)",
   "Framebuffer (fbfe)",
+  "Wayland (wlfe)",
   trNOOP("Off"),
   NULL 
 };
@@ -171,6 +172,7 @@ const char * const config_t::s_frontendNames[ FRONTEND_count+1 ] = {
 const char * const config_t::s_frontend_files[ FRONTEND_count+1 ] = {
   "lib" PLUGIN_NAME_I18N "-sxfe.so." XINELIBOUTPUT_VERSION,
   "lib" PLUGIN_NAME_I18N "-fbfe.so." XINELIBOUTPUT_VERSION,
+  "lib" PLUGIN_NAME_I18N "-wlfe.so." XINELIBOUTPUT_VERSION,
   // example: libxineliboutput-sxfe.so.0.4.0
   "",
   NULL
@@ -388,11 +390,17 @@ static char *strcatrealloc(char *dest, const char *src)
 
   size_t l = (dest ? strlen(dest) : 0) + strlen(src) + 1;
   if(dest) {
-    dest = (char *)realloc(dest, l);
+    char *tmp = (char *)realloc(dest, l);
+    if (!tmp) {
+      free(dest);
+      return NULL;
+    }
+    dest = tmp;
     strcat(dest, src);
   } else {
     dest = (char*)malloc(l);
-    strcpy(dest, src);
+    if (dest)
+      strcpy(dest, src);
   } 
   return dest;
 }
@@ -815,7 +823,7 @@ bool config_t::ProcessArgs(int argc, char *argv[])
     case 'h': //ProcessArg("Fullscreen", "0");
               ProcessArg("X11.WindowHeight", optarg);
               break;
-    case 'g': {
+    case 'g': if (optarg) {
                 int _width = width, _height = height, _xpos = 0, _ypos = 0;
                 sscanf (optarg, "%dx%d+%d+%d", &_width, &_height, &_xpos, &_ypos);
                 ProcessArg("X11.WindowWidth",  *cString::sprintf("%d", _width));
@@ -832,7 +840,7 @@ bool config_t::ProcessArgs(int argc, char *argv[])
               break;
     //case 'm': ProcessArg("Modeline", optarg);
     //          break;
-    case 'r': if(strcmp(optarg, "none")) {
+    case 'r': if (optarg && strcmp(optarg, "none")) {
                 if(strchr(optarg, ':')) {
 		  char *tmp = strdup(optarg);
 		  char *pt = strchr(tmp,':');
@@ -851,7 +859,7 @@ bool config_t::ProcessArgs(int argc, char *argv[])
               break;
     case 'V': ProcessArg("Video.Driver", optarg);
               break;
-    case 'A': if(strchr(optarg,':')) {
+    case 'A': if (optarg && strchr(optarg,':')) {
                 char *tmp = strdup(optarg);
 		char *pt = strchr(tmp,':');
 		*pt = 0;
@@ -865,7 +873,10 @@ bool config_t::ProcessArgs(int argc, char *argv[])
                 post_plugins = strcatrealloc(post_plugins, ";");
               post_plugins = strcatrealloc(post_plugins, optarg);
               break;
-    case 'C': config_file = strdup(optarg);
+    case 'C': if (optarg) {
+                free(config_file);
+                config_file = strdup(optarg);
+              }
               break;
     case 'p': ProcessArg("ForcePrimaryDevice", "1");
               break;
