@@ -44,6 +44,9 @@
 
 #include "device.h"
 
+template<class T> inline T min_(T a, T b) { return a < b ? a : b; }
+template<class T> inline T max_(T a, T b) { return a > b ? a : b; }
+
 #define STILLPICTURE_REPEAT_COUNT 3
 #define LOCAL_INIT_TIMEOUT        20  // seconds
 #define SERVER_INIT_TIMEOUT       5   // seconds
@@ -1352,7 +1355,7 @@ bool cXinelibDevice::Poll(cPoller &Poller, int TimeoutMs)
     return true;
 
   if(m_TrickSpeed == 0) {
-    cCondWait::SleepMs(min(TimeoutMs, 20));
+    cCondWait::SleepMs(min_(TimeoutMs, 20));
     return Poller.Poll(0);
   }
 
@@ -1367,11 +1370,11 @@ bool cXinelibDevice::Poll(cPoller &Poller, int TimeoutMs)
     int result = DEFAULT_POLL_SIZE;
 
     if(m_local)
-      result = min(result, m_local->Poll(Poller, TimeoutMs));
+      result = min_(result, m_local->Poll(Poller, TimeoutMs));
     if(m_server)
-      result = min(result, m_server->Poll(Poller, TimeoutMs));
+      result = min_(result, m_server->Poll(Poller, TimeoutMs));
 
-    m_FreeBufs = max(result, 0);
+    m_FreeBufs = max_(result, 0);
   }
 
   return m_FreeBufs > 0 /*|| Poller.Poll(0)*/;
@@ -1523,16 +1526,18 @@ void cXinelibDevice::GetOsdSize(int &Width, int &Height, double &PixelAspect)
       break;
     case OSD_SIZE_stream:
       if (m_VideoSize->width > 0 && m_VideoSize->height > 0) {
-        Width  = max<int>(m_VideoSize->width,  512);
-        Height = max<int>(m_VideoSize->height, 480);
+        Width  = max_<int>(m_VideoSize->width,  512);
+        Height = max_<int>(m_VideoSize->height, 480);
         break;
       }
+      /* fall through */
     case OSD_SIZE_auto:
       if (xc.osd_width_auto > 0 && xc.osd_height_auto > 0) {
         Width  = xc.osd_width_auto;
         Height = xc.osd_height_auto;
         break;
       }
+      /* fall through */
     case OSD_SIZE_custom:
     default:
       Width  = xc.osd_width;
