@@ -41,6 +41,7 @@
 #include "tools/general_remote.h"
 #include "tools/iso639.h"
 
+template<class T> inline T max_(T a, T b) { return a > b ? a : b; }
 
 #ifndef STARTUP_IMAGE_FILE
 #  define  STARTUP_IMAGE_FILE "/usr/share/vdr/xineliboutput/logo.mpv"
@@ -424,7 +425,7 @@ int cXinelibThread::Poll(cPoller& Poller, int TimeoutMs)
 
   int n = Xine_Control("POLL", TimeoutMs);
 
-  return max(n, 0);
+  return max_(n, 0);
 }
 
 //
