@@ -192,8 +192,9 @@ void cXinelibOsd::CmdSize(int Width, int Height)
   TRACEF("cXinelibOsd::CmdSize");
 
   if (m_Device && m_WindowHandles) {
-    osd_command_t osdcmd = {};
+    osd_command_t osdcmd;
 
+    memset(&osdcmd, 0, sizeof(osdcmd));
     osdcmd.cmd = OSD_Size;
     osdcmd.w   = Width;
     osdcmd.h   = Height;
@@ -214,10 +215,11 @@ void cXinelibOsd::CmdVideoWindow(int X, int Y, int W, int H)
   TRACEF("cXinelibOsd::CmdVideoWindow");
 
   if (m_Device && m_WindowHandles) {
-    osd_command_t osdcmd = {};
+    osd_command_t osdcmd;
 
+    memset(&osdcmd, 0, sizeof(osdcmd));
     for (int Wnd = 0; m_WindowHandles[Wnd] >= 0; Wnd++) {
-      osdcmd.cmd = OSD_VideoWindow;
+      osdcmd.cmd = OSD_VideoWindow,
       osdcmd.wnd = m_WindowHandles[Wnd];
       osdcmd.x   = X;
       osdcmd.y   = Y;
@@ -234,8 +236,9 @@ void cXinelibOsd::CmdClose(int Wnd)
   TRACEF("cXinelibOsd::CmdClose");
 
   if (m_Device) {
-    osd_command_t osdcmd = {};
+    osd_command_t osdcmd;
 
+    memset(&osdcmd, 0, sizeof(osdcmd));
     osdcmd.cmd = OSD_Close;
     osdcmd.wnd = m_WindowHandles[Wnd];
 
@@ -253,8 +256,9 @@ void cXinelibOsd::CmdFlush(void)
   TRACEF("cXinelibOsd::CmdFlush");
 
   if (m_Device) {
-    osd_command_t osdcmd = {};
+    osd_command_t osdcmd;
 
+    memset(&osdcmd, 0, sizeof(osdcmd));
     osdcmd.cmd = OSD_Flush;
 
     m_Device->OsdCmd((void*)&osdcmd);
@@ -285,8 +289,9 @@ void cXinelibOsd::CmdArgb(int X, int Y, int W, int H,
 
   if (m_Device) {
 
-    osd_command_t osdcmd = {};
+    osd_command_t osdcmd;
 
+    memset(&osdcmd, 0, sizeof(osdcmd));
     osdcmd.cmd   = OSD_Set_ARGB;
     osdcmd.wnd   = m_WindowHandles[0];
     osdcmd.layer = saturate(m_Layer, 0, 0xffff);
@@ -318,8 +323,9 @@ void cXinelibOsd::CmdLut8(int Wnd, int X0, int Y0,
   if (m_Device) {
 
     osd_clut_t    clut[Colors];
-    osd_command_t osdcmd = {};
+    osd_command_t osdcmd;
 
+    memset(&osdcmd, 0, sizeof(osdcmd));
     osdcmd.cmd   = OSD_Set_LUT8;
     osdcmd.wnd   = m_WindowHandles[Wnd];
     osdcmd.layer = saturate(m_Layer, 0, 0xffff);
