@@ -20,7 +20,6 @@
 
 #ifndef LOG_MODULENAME
 #  define LOG_MODULENAME "[mpeg-ts  ] "
-#  define SysLogLevel    iSysLogLevel
 #  include "../logdefs.h"
 #endif
 
@@ -376,6 +375,7 @@ int ts_parse_pmt (pmt_data_t *pmt, unsigned program_no, const uint8_t *pkt)
   pmt->audio_tracks_count = 0;
   pmt->spu_tracks_count = 0;
   pmt->video_pid = INVALID_PID;
+  pmt->video_type = STREAM_NONE;
 
   /* ES definitions start here */
   program_info_length = ((pmt->pmt[10] << 8) | pmt->pmt[11]) & 0x0fff;
@@ -634,7 +634,7 @@ ts_state_t *ts_state_init(size_t buffer_size)
   if (buffer_size < 8 * TS_SIZE)
     buffer_size = 8 * TS_SIZE;
   if (buffer_size > 4*1024*1024) {
-    LOGMSG("ERROR: ts_state_init(%zd)", buffer_size);
+    LOGMSG("ERROR: ts_state_init(%zu)", buffer_size);
     buffer_size = 4*1024*1024;
   }
 
@@ -677,6 +677,11 @@ static size_t ts_add_payload(ts_state_t *ts, const uint8_t *data)
     ts->buf_len   = 0;
   }
 
+  if (ts->buf_size < 2*TS_SIZE) {
+    LOGMSG("ts_add_payload(): assertion failed: buf_size < 2*TS_SIZE");
+    return 0;
+  }
+
   if (ts->buf_len >= ts->buf_size - TS_SIZE) {
     LOGDBG("ts_add_payload: buffer full");
     ts->buf_len -= TS_SIZE;
