@@ -15,6 +15,8 @@
 extern "C" {
 #endif
 
+#include <stdint.h>
+#include <stddef.h>
 
 /* Avoid warnings when included to VDR plugin */
 #undef TS_SYNC_BYTE
@@ -100,6 +102,7 @@ typedef enum {
   HDMV_SPU_91_INTERACTIVE = 0x91 | STREAM_HDMV, /* Interactive Graphics */
   HDMV_SPU_92_TEXT        = 0x92 | STREAM_HDMV, /* Text subtitles */
 
+  STREAM_NONE             = 0,
 } ts_stream_type;
 
 /* stream info descriptors */
@@ -170,6 +173,7 @@ typedef struct {
     uint16_t       aux_page_id;
   } spu_tracks[TS_MAX_SPU_TRACKS];
 
+  uint8_t raw_pmt[4096];
 } pmt_data_t;
 
 /*
@@ -199,7 +203,9 @@ void        ts_state_reset(ts_state_t *ts);
 void        ts_state_dispose(ts_state_t *ts);
 
 int64_t ts_get_pts(ts_state_t *ts, const uint8_t *data);
+/*
 int     ts_get_picture_type(ts_state_t *ts, const uint8_t *data, int h264);
+*/
 int     ts_get_video_size(ts_state_t *ts, const uint8_t *data, struct video_size_s *size, ts_stream_type vid_type);
 
 
