@@ -99,14 +99,14 @@ cUdpScheduler::cUdpScheduler()
 
   // RTP
 
-  srandom(time(NULL) ^ getpid());
-
-  m_ssrc = random();
+  struct timespec ts = { 0, 0 };
+  clock_gettime(CLOCK_REALTIME, &ts);
+  m_ssrc = ts.tv_nsec ^ getpid();
   LOGDBG("RTP SSRC: 0x%08x", m_ssrc);
   m_LastRtcpTime = 0;
   m_Frames = 0;
   m_Octets = 0;
-  m_RtpScr.Set((int64_t)random());
+  m_RtpScr.Set(ts.tv_nsec);
 
   m_fd_sap = -1;
 
@@ -163,7 +163,9 @@ bool cUdpScheduler::AddRtp(void)
   }
 
   /* need new ssrc */
-  m_ssrc = random();
+  struct timespec ts = { 0, 0 };
+  clock_gettime(CLOCK_REALTIME, &ts);
+  m_ssrc ^= ts.tv_nsec;
   LOGDBG("RTP SSRC: 0x%08x", m_ssrc);
 
   //
@@ -184,14 +186,17 @@ bool cUdpScheduler::AddRtp(void)
   }
 
   if(xc.remote_local_ip[0]) {
-    struct sockaddr_in name;
-    memset(&name, 0, sizeof(name));
-    name.sin_family = AF_INET;
-    name.sin_addr.s_addr = inet_addr(xc.remote_local_ip);
-    if(name.sin_addr.s_addr == INADDR_NONE)
+    union {
+      struct sockaddr    sa;
+      struct sockaddr_in in;
+    } addr;
+    memset(&addr, 0, sizeof(addr));
+    addr.in.sin_family = AF_INET;
+    addr.in.sin_addr.s_addr = inet_addr(xc.remote_local_ip);
+    if(addr.in.sin_addr.s_addr == INADDR_NONE)
       LOGERR("Local address %s is invalid", xc.remote_local_ip);
-    name.sin_port = htons(xc.remote_rtp_port);
-    if (bind(m_fd_rtp.handle(), (struct sockaddr *)&name, sizeof(name)) < 0)
+    addr.in.sin_port = htons(xc.remote_rtp_port);
+    if (bind(m_fd_rtp.handle(), &addr.sa, sizeof(addr)) < 0)
       LOGERR("bind(%s:%d) failed for udp/rtp multicast", xc.remote_local_ip, xc.remote_rtp_port);
 #if 0
     struct ip_mreqn mreqn;
@@ -226,12 +231,15 @@ bool cUdpScheduler::AddRtp(void)
     m_fd_rtcp.close();
 
   if(xc.remote_local_ip[0]) {
-    struct sockaddr_in name;
-    memset(&name, 0, sizeof(name));
-    name.sin_family = AF_INET;
-    name.sin_addr.s_addr = inet_addr(xc.remote_local_ip);
-    name.sin_port = htons(xc.remote_rtp_port+1);
-    if (bind(m_fd_rtcp.handle(), (struct sockaddr *)&name, sizeof(name)) < 0)
+    union {
+      struct sockaddr    sa;
+      struct sockaddr_in in;
+    } addr;
+    memset(&addr, 0, sizeof(addr));
+    addr.in.sin_family = AF_INET;
+    addr.in.sin_addr.s_addr = inet_addr(xc.remote_local_ip);
+    addr.in.sin_port = htons(xc.remote_rtp_port+1);
+    if (bind(m_fd_rtcp.handle(), &addr.sa, sizeof(addr)) < 0)
       LOGERR("bind(%s:%d) failed for udp/rtp multicast", xc.remote_local_ip, xc.remote_rtp_port);
 #if 0
     struct ip_mreqn mreqn;
@@ -432,7 +440,7 @@ void cUdpScheduler::SetScrSpeed(const int Speed)
   m_RtpScr.SetScrSpeed(Speed);
 }
 
-bool cUdpScheduler::Queue(eStreamId StreamId, uint64_t StreamPos, const uchar *Data, int Length)
+bool cUdpScheduler::Queue(eStreamId StreamId, uint64_t StreamPos, const uint8_t *Data, int Length)
 {
   cMutexLock ml(&m_Lock);
 
@@ -585,7 +593,7 @@ void cUdpScheduler::Send_RTCP(void)
     msg->hdr.length = htons(1 + 1 + ((it->length - 2) + 3) / 4);
 
     content += sizeof(rtcp_common_t) + 4*ntohs(msg->hdr.length);
-    msg = (rtcp_packet_t *)content;
+    //msg = (rtcp_packet_t *)content;
 
     // Send
 #ifndef LOG_RTCP
@@ -650,15 +658,14 @@ void cUdpScheduler::Send_SAP(bool Announce)
   }
 #endif
 
-  sap_pdu_t *pdu = sap_create_pdu(local_addr,
-				  Announce, 
-				  (m_ssrc >> 16 | m_ssrc) & 0xffff,
-				  "application/sdp",
-				  (const char *)sdp_descr);
-      
-  if(!sap_send_pdu(&m_fd_sap, pdu, 0))
+  if (!sap_send_announce(&m_fd_sap, 0,
+                         local_addr,
+                         (m_ssrc >> 16 | m_ssrc) & 0xffff,
+                         Announce,
+                         "application/sdp",
+                         (const char *)sdp_descr)) {
     LOGERR("SAP/SDP announce failed");
-  free(pdu);
+  }
 
   if(!Announce)
     CLOSESOCKET(m_fd_sap);
@@ -668,7 +675,7 @@ void cUdpScheduler::Send_SAP(bool Announce)
 static const char ScrSourceName[][6] = {"???", "VIDEO", "PS1", "AUDIO", "PCR"};
 #endif
 
-void cUdpScheduler::Schedule(const uchar *Data, int Length)
+void cUdpScheduler::Schedule(const uint8_t *Data, int Length)
 {
   int64_t     pts       = NO_PTS;
   int         elapsed   = 0;
@@ -887,7 +894,9 @@ void cUdpScheduler::ReSend(int fd, uint64_t Pos, int Seq1, int Seq2)
     snprintf(udp_ctrl.payload, sizeof(udp_ctrl.payload),
              "UDP MISSING %d-%d %" PRIu64,
              Seq1, (Seq2 & UDP_BUFFER_MASK), Pos);
-    send(fd, &udp_ctrl, sizeof(udp_ctrl), 0);
+    if (send(fd, &udp_ctrl, sizeof(udp_ctrl), 0) != sizeof(udp_ctrl)) {
+      LOGERR("cUdpScheduler: UDP/RTP re-send() failed !");
+    }
     return;
   }
 
@@ -906,12 +915,13 @@ void cUdpScheduler::ReSend(int fd, uint64_t Pos, int Seq1, int Seq2)
 
     if(frame) {
       if(priv_ntohull(frame->hdr_ext.pos) - Pos < 100000) {
-	send(fd,
-	     RTP_UDP_PAYLOAD(frame),
-	     m_BackLog->PayloadSize(Seq1) + sizeof(stream_udp_header_t),
-	     0);
-	LOGRESEND("cUdpScheduler::ReSend: %d (%d bytes) @%lld sent",
-		  Seq1, m_BackLog->PayloadSize(Seq1), Pos);
+        ssize_t pac_len = m_BackLog->PayloadSize(Seq1) + sizeof(stream_udp_header_t);
+        if (send(fd, RTP_UDP_PAYLOAD(frame), pac_len, 0) != pac_len) {
+          LOGERR("cUdpScheduler: UDP/RTP re-send() failed !");
+        } else {
+          LOGRESEND("cUdpScheduler::ReSend: %d (%d bytes) @%lld sent",
+                    Seq1, m_BackLog->PayloadSize(Seq1), Pos);
+        }
 	Pos = priv_ntohull(frame->hdr_ext.pos) + m_BackLog->PayloadSize(Seq1);
 	continue;
       } else {
@@ -939,6 +949,8 @@ void cUdpScheduler::ReSend(int fd, uint64_t Pos, int Seq1, int Seq2)
              "UDP MISSING %d-%d %" PRIu64,
              Seq0, (Seq1 & UDP_BUFFER_MASK), Pos);
 
-    send(fd, &udp_ctrl, sizeof(udp_ctrl), 0);
+    if (send(fd, &udp_ctrl, sizeof(udp_ctrl), 0) != sizeof(udp_ctrl)) {
+      LOGERR("cUdpScheduler: UDP/RTP control send() failed !");
+    }
   }
 }
