@@ -36,16 +36,47 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include <xine.h>
+
 #define LOG_MODULENAME "[xine-post] "
 #include "../logdefs.h"
 
 #define fe_t post_plugins_t
 
+
+#ifdef _WIN32
+static char *strsep(char **stringp, const char *delim)
+{
+  char *s;
+  const char *spanp;
+  int c, sc;
+  char *tok;
+
+  if ((s = *stringp) == NULL)
+    return (NULL);
+  for (tok = s;;) {
+    c = *s++;
+    spanp = delim;
+    do {
+      if ((sc = *spanp++) == c) {
+        if (c == 0)
+          s = NULL;
+        else
+          s[-1] = 0;
+        *stringp = s;
+        return (tok);
+      }
+    } while (sc != 0);
+  }
+    /* NOTREACHED */
+}
+#endif
+
 typedef struct {
   xine_post_t                 *post;
-  xine_post_api_t             *api;
-  xine_post_api_descr_t       *descr;
-  xine_post_api_parameter_t   *param;
+  const xine_post_api_t           *api;
+  const xine_post_api_descr_t     *descr;
+  const xine_post_api_parameter_t *param;
   char                        *param_data;
 
   int                          x;
@@ -59,16 +90,16 @@ typedef struct {
 
 static int __pplugin_retrieve_parameters(post_object_t *pobj)
 {
-  xine_post_in_t             *input_api;
+  const xine_post_in_t *input_api;
 
-  if((input_api = (xine_post_in_t *) xine_post_input(pobj->post,
-                                                     "parameters"))) {
-    xine_post_api_t            *post_api;
-    xine_post_api_descr_t      *api_descr;
-    xine_post_api_parameter_t  *parm;
-    int                         pnum = 0;
+  input_api = (const xine_post_in_t *) xine_post_input(pobj->post, "parameters");
+  if (input_api) {
+    const xine_post_api_t            *post_api;
+    const xine_post_api_descr_t      *api_descr;
+    const xine_post_api_parameter_t  *parm;
+    int                               pnum = 0;
 
-    post_api = (xine_post_api_t *) input_api->data;
+    post_api = /*(const xine_post_api_t *)*/ input_api->data;
 
     api_descr = post_api->get_param_descr();
 
@@ -115,12 +146,11 @@ static void __pplugin_update_parameters(xine_post_t *post, char *args)
   };
 
   if(__pplugin_retrieve_parameters(&pobj)) {
-    int   i;
 
     if(pobj.properties_names && args && *args) {
       char *param;
 
-      while((param = xine_strsep(&args, ",")) != NULL) {
+      while ((param = strsep(&args, ",")) != NULL) {
 
         p = param;
 
@@ -211,7 +241,7 @@ static void __pplugin_update_parameters(xine_post_t *post, char *args)
     }
 
     if (pobj.properties_names) {
-      i = 0;
+      int i = 0;
 
       while(pobj.properties_names[i]) {
         free(pobj.properties_names[i]);
@@ -243,7 +273,7 @@ static post_element_t **pplugin_parse_and_load(fe_t *fe,
 
     freeme = post_chain = strdup(pchain);
 
-    while((p = xine_strsep(&post_chain, ";"))) {
+    while ((p = strsep(&post_chain, ";"))) {
 
       if(p && strlen(p)) {
         char          *plugin, *args = NULL;
@@ -308,6 +338,21 @@ static post_element_t **pplugin_parse_and_load(fe_t *fe,
   return post_elements;
 }
 
+static void strcat_n(size_t sz, char *dst, const char *src)
+{
+  size_t len = strlen(dst) + strlen(src);
+  if (sz < 1)
+    return;
+  if (len + 6 > sz) {
+    while (len + 1 < sz) {
+      dst[len++] = '.';
+    }
+    dst[sz-1] = 0;
+    return;
+  }
+  strcat(dst, src);
+}
+
 static
 void pplugin_parse_and_store_post(fe_t *fe, int plugin_type,
                                   const char *post_chain)
@@ -364,12 +409,12 @@ void pplugin_parse_and_store_post(fe_t *fe, int plugin_type,
         if((*_post_elements)[i])
           if(((*_post_elements)[i])->post) {
             if(((*_post_elements)[i])->enable)
-              strcat(s, "*");
+              strcat_n(sizeof(s), s, "*");
             if(((*_post_elements)[i])->name)
-              strcat(s, ((*_post_elements)[i])->name);
+              strcat_n(sizeof(s), s, ((*_post_elements)[i])->name);
             else
-              strcat(s, "<no name!>");
-            strcat(s, " ");
+              strcat_n(sizeof(s), s, "<no name!>");
+            strcat_n(sizeof(s), s, " ");
             }
       LOGDBG("    loaded plugins (type %d.%d): %s",
              (plugin_type>>16), (plugin_type&0xffff), s);
