@@ -8,6 +8,8 @@
  *
  */
 
+#include "vo_lastpts.h"
+
 #include <stdlib.h>
 
 #include <xine/xine_internal.h>
@@ -21,8 +23,6 @@
 #define LOG_MODULENAME "[lastpts  ] "
 #include "../logdefs.h"
 
-#include "vo_lastpts.h"
-
 /*
  *  lastpts_hook_t
  */
@@ -100,6 +100,9 @@ vo_driver_t *vo_lastpts_init(void)
 {
   lastpts_hook_t *this = calloc(1, sizeof(lastpts_hook_t));
 
+  if (!this)
+    return NULL;
+
   this->h.vo.display_frame = lastpts_display_frame;
   this->h.vo.dispose       = vo_def_dispose;
 
