@@ -8,10 +8,13 @@
  *
  */
 
+#include "features.h"
+
 #include <inttypes.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <pthread.h>
 
 #if defined(__linux__)
 # include <linux/kd.h>
@@ -40,6 +43,7 @@ typedef struct fbfe_s {
                       const char *audio_port,
                       const char *video_driver,
                       int pes_buffers,
+                      int ffmpeg_threads,
                       const char *static_post_plugins,
                       const char *config_file);
 
@@ -57,12 +61,15 @@ static void fbfe_update_display_size(fe_t *this_gen)
 {
   fbfe_t *this = (fbfe_t*)this_gen;
   if(this->fullscreen && this->x.video_port) {
-    this->x.width  = this->x.video_port->get_property(this->x.video_port, 
-						      VO_PROP_WINDOW_WIDTH);
-    this->x.height = this->x.video_port->get_property(this->x.video_port, 
-						      VO_PROP_WINDOW_HEIGHT);
-    LOGDBG("Framebuffer size after initialization: %dx%d",
-	   this->x.width, this->x.height);
+    int w, h;
+    w = this->x.video_port->get_property(this->x.video_port, VO_PROP_WINDOW_WIDTH);
+    h = this->x.video_port->get_property(this->x.video_port,  VO_PROP_WINDOW_HEIGHT);
+    LOGDBG("Framebuffer size after initialization: %dx%d", w, h);
+
+    pthread_mutex_lock(&this->x.geom_mutex);
+    this->x.width = w;
+    this->x.height = h;
+    pthread_mutex_unlock(&this->x.geom_mutex);
   }
 }
 
@@ -76,7 +83,7 @@ static void update_DFBARGS(const char *fb_dev)
   const char *env_old = getenv("DFBARGS");
   char *env_new = NULL;
 
-  if (env_old) {
+  if (env_old && env_old[0]) {
     char *env_tmp = strdup(env_old);
     char *head    = strstr(env_tmp, "fbdev=");
 
@@ -92,7 +99,7 @@ static void update_DFBARGS(const char *fb_dev)
         return;
       }
     } else {
-      if(asprintf(&env_new, "fbdev=%s%s%s", fb_dev, env_tmp ? "," : "", env_tmp ?: "") < 0) {
+      if (asprintf(&env_new, "fbdev=%s,%s", fb_dev, env_tmp) < 0) {
         free(env_tmp);
         return;
       }
@@ -139,14 +146,16 @@ static int fbfe_display_open(frontend_t *this_gen,
   LOGDBG("fbfe_display_open(width=%d, height=%d, fullscreen=%d, display=%s)",
 	 width, height, fullscreen, video_port);
 
+  pthread_mutex_lock(&this->x.geom_mutex);
   this->x.xpos          = xpos;
   this->x.ypos          = ypos;
   this->x.width         = width;
   this->x.height        = height;
+  pthread_mutex_unlock(&this->x.geom_mutex);
   this->x.aspect        = aspect;
 /*this->x.cropping      = 0;*/
   this->x.scale_video   = scale_video;
-  this->x.overscan      = 0;
+/*this->x.overscan      = 0;*/
   this->x.display_ratio = 1.0;
   this->x.aspect_controller = aspect_controller ? strdup(aspect_controller) : NULL;
 
@@ -200,10 +209,13 @@ static int fbfe_display_config(frontend_t *this_gen,
   if(!this)
     return 0;
 
+  pthread_mutex_lock(&this->x.geom_mutex);
   this->x.xpos        = xpos   >= 0 ? xpos   : this->x.xpos;
   this->x.ypos        = ypos   >= 0 ? ypos   : this->x.ypos;
   this->x.width       = width  >= 0 ? width  : this->x.width;
   this->x.height      = height >= 0 ? height : this->x.height;
+  pthread_mutex_unlock(&this->x.geom_mutex);
+
   this->x.aspect      = aspect;
   this->x.scale_video = scale_video;
   this->fullscreen    = fullscreen;
@@ -271,6 +283,7 @@ static int fbfe_xine_init(frontend_t *this_gen, const char *audio_driver,
                           const char *audio_port,
                           const char *video_driver,
                           int pes_buffers,
+                          int ffmpeg_threads,
                           const char *static_post_plugins,
                           const char *config_file)
 {
@@ -280,13 +293,16 @@ static int fbfe_xine_init(frontend_t *this_gen, const char *audio_driver,
     update_DFBARGS(this->x.video_port_name);
 
   return this->fe_xine_init(this_gen, audio_driver, audio_port,
-                            video_driver, pes_buffers, static_post_plugins, config_file);
+                            video_driver, pes_buffers, ffmpeg_threads, static_post_plugins, config_file);
 }
 
 static frontend_t *fbfe_get_frontend(void)
 {
   fbfe_t *this = calloc(1, sizeof(fbfe_t));
 
+  if (!this)
+    return NULL;
+
   this->fd_tty = -1;
 
   init_fe((fe_t*)this);
