@@ -15,6 +15,8 @@
 extern "C" {
 #endif
 
+#include <stdint.h>
+
 #define FE_VERSION_STR  XINELIBOUTPUT_VERSION /*"1.0.0pre1"*/
 
 typedef struct frontend_config_s frontend_config_t;
@@ -98,6 +100,7 @@ struct frontend_s {
                     const char *audio_port,
                     const char *video_driver,
                     int pes_buffers,
+                    int ffmpeg_threads,
                     const char *static_post,
                     const char *config_file);
   int  (*xine_open)(frontend_t*, const char *mrl);
