@@ -11,13 +11,18 @@
 #ifndef XINE_FRONTEND_INTERNAL_H
 #define XINE_FRONTEND_INTERNAL_H
 
+#include <sys/types.h>  // pthread_mutex_t
+
 #include <xine.h>
 #include <xine/input_plugin.h>
+#include <xine/video_out.h>
 
 #include "xine_frontend.h"
 #include "xine_input_vdr.h"
 #include "xine/post.h"
 
+#include "tools/atomic.h"
+
 typedef struct fe_s {
   /* base class */
   frontend_t          fe;
@@ -65,25 +70,31 @@ typedef struct fe_s {
     raw_visual_t      vis_raw;
 #ifdef _WIN32
     win32_visual_t    vis_win;
+#endif
+#ifdef XINE_VISUAL_TYPE_WAYLAND
+    xine_wayland_visual_t vis_wl;
 #endif
   };
 
   /* frontend */
   double      video_aspect;    /* aspect ratio of video frame */
   double      display_ratio;   /* aspect ratio of video window */
-  uint8_t     terminate_key_pressed;
+  uint16_t    video_width;     /* size of video frame */
+  uint16_t    video_height;    /* */
+  int         pes_buffers;     /* max. number of PES packets in video fifo. -1 = do not change. */
+  uint8_t     aspect;          /* aspect ratio of video window (user setting) */
+/*uint8_t     overscan;*/      /* overscan in % (crop video borders) */
+/*uint8_t     cropping : 1;*/
+  uint8_t     scale_video : 1; /* enable/disable all video scaling */
+  uint8_t     slave_playback_finished : 1;
+
+  xl_atomic_int playback_finished;
+  xl_atomic_int terminate_key_pressed;
+
+  pthread_mutex_t geom_mutex;
   uint16_t    xpos, ypos;      /* position of video window */
   uint16_t    width;           /* size of video window */
   uint16_t    height;          /* */
-  uint16_t    video_width;     /* size of video frame */
-  uint16_t    video_height;    /* */
-  uint16_t    pes_buffers;     /* max. number of PES packets in video fifo */
-  uint8_t     aspect;          /* aspect ratio of video window (user setting) */
-  uint8_t     overscan;        /* overscan in % (crop video borders) */
-/*uint8_t     cropping : 1;*/
-  uint8_t     scale_video : 1; /* enable/disable all video scaling */
-  uint8_t     playback_finished : 1;
-  uint8_t     slave_playback_finished : 1;
 
   char       *shutdown_cmd;
   int         shutdown_timeout;
