@@ -19,7 +19,9 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#ifndef _WIN32
 #include <syslog.h>
+#endif
 #include <getopt.h>
 #include <signal.h>
 
@@ -28,9 +30,10 @@
 #define LOG_MODULENAME "[vdr-fe]    "
 #include "logdefs.h"
 
-#include "xine_input_vdr_mrl.h"
 #include "xine_frontend.h"
+#include "xine/input_xvdr_mrl.h"
 #include "tools/vdrdiscovery.h"
+
 #include "xine_frontend_cec.h"
 #include "xine_frontend_lirc.h"
 #include "xine_frontend_kbd.h"
@@ -40,12 +43,16 @@
 /* next symbol is dynamically linked from input plugin */
 int SysLogLevel __attribute__((visibility("default"))) = SYSLOGLEVEL_INFO; /* errors and info, no debug */
 
+#ifdef _WIN32
+static const int last_signal = 0;
+#else
 volatile int   last_signal = 0;
-int            gui_hotkeys = 0;
+#endif
 
 /*
  * SignalHandler()
  */
+#ifndef _WIN32
 static void SignalHandler(int signum)
 {
   LOGMSG("caught signal %d", signum);
@@ -66,6 +73,7 @@ static void SignalHandler(int signum)
 
   signal(signum, SignalHandler);
 }
+#endif
 
 /*
  * strcatrealloc()
@@ -140,6 +148,7 @@ static const char help_str[] =
     "   -g, --geometry=WxH[+X+Y]      Set output window geometry (X style)\n"
 #endif // !IS_FBFE
     "   -B, --buffers=x               Number of PES buffers\n"
+    "       --threads=x               Number of ffmpeg decoding threads (default: auto)\n"
     "   -n, --noscaling               Disable all video scaling\n"
     "   -P, --post=name[:arg=val[,arg=val]]\n"
     "                                 Load and use xine post plugin(s)\n"
@@ -157,7 +166,9 @@ static const char help_str[] =
     "   -C, --config=file             Use config file (default: ~/.xine/config_xineliboutput).\n"
     "   -v, --verbose                 Verbose debug output\n"
     "   -s, --silent                  Silent mode (report only errors)\n"
+#ifndef _WIN32
     "   -l, --syslog                  Write all output to system log\n"
+#endif
     "   -k, --nokbd                   Disable console keyboard input\n"
 #ifndef IS_FBFE
     "   -x, --noxkbd                  Disable X11 keyboard input\n"
@@ -171,6 +182,7 @@ static const char help_str[] =
     "                                 log to syslog and fork to background)\n"
     "   -S, --slave                   Enable slave mode (read commands from stdin)\n"
     "   -R, --reconnect               Automatically reconnect when connection has been lost\n"
+    "   -c, --waitvdr                 Wait for VDR to start (retry connecting)\n"
     "   -t, --tcp                     Use TCP transport\n"
     "   -u, --udp                     Use UDP transport\n"
     "   -r, --rtp                     Use RTP transport\n\n"
@@ -203,6 +215,7 @@ static const struct option long_options[] = {
   { "height",     required_argument, NULL, 'h' },
 #endif
   { "buffers",    required_argument, NULL, 'B' },
+  { "threads",    required_argument, NULL,  10 },
   { "noscaling",  no_argument,       NULL, 'n' },
   { "post",       required_argument, NULL, 'P' },
   { "lirc",       optional_argument, NULL, 'L' },
@@ -216,7 +229,9 @@ static const struct option long_options[] = {
 
   { "verbose", no_argument,  NULL, 'v' },
   { "silent",  no_argument,  NULL, 's' },
+#ifndef _WIN32
   { "syslog",  no_argument,  NULL, 'l' },
+#endif
   { "nokbd",   no_argument,  NULL, 'k' },
   { "noxkbd",  no_argument,  NULL, 'x' },
   { "hotkeys", no_argument,  NULL, 'o' },
@@ -225,6 +240,7 @@ static const struct option long_options[] = {
   { "slave",   no_argument,  NULL, 'S' },
 
   { "reconnect", no_argument,  NULL, 'R' },
+  { "waitvdr",   no_argument,  NULL, 'c' },
   { "tcp",       no_argument,  NULL, 't' },
   { "udp",       no_argument,  NULL, 'u' },
   { "rtp",       no_argument,  NULL, 'r' },
@@ -236,18 +252,21 @@ static const struct option long_options[] = {
 
 int main(int argc, char *argv[])
 {
-  int ftcp = 0, fudp = 0, frtp = 0, reconnect = 0, firsttry = 1;
+  int ftcp = 0, fudp = 0, frtp = 0, reconnect = 0, firsttry = 1, waitvdr = 0;
   int fullscreen = 0, hud = 0, opengl = 0, xpos = 0, ypos = 0, width = 720, height = 576;
-  int pes_buffers = 250;
+  int pes_buffers = -1;
+  int ffmpeg_threads = 0;
   int scale_video = 1, aspect = 1, modeswitch = 0;
   int daemon_mode = 0, nokbd = 0, noxkbd = 0, slave_mode = 0;
   int repeat_emu = 0;
+  int gui_hotkeys = 0;
   int touchscreen = 0;
   int window_id = WINDOW_ID_NONE;
   int xmajor, xminor, xsub;
   int c;
   int xine_finished = FE_XINE_ERROR;
   int inactivity_timer = 0;
+  int result = 0;
   char *mrl = NULL;
   char *video_driver = NULL;
   char *audio_driver = NULL;
@@ -263,11 +282,23 @@ int main(int argc, char *argv[])
   const char *config_file = NULL;
   const char *power_off_cmd = NULL;
 
+  input_cec_t  *cec = NULL;
+  input_kbd_t  *kbd = NULL;
+  input_lirc_t *lirc = NULL;
+
   extern const fe_creator_f fe_creator;
   frontend_t *fe = NULL;
 
   LogToSysLog = 0;
 
+  const char *lib_verbosity = getenv("LIBXINE_VERBOSITY");
+  if (lib_verbosity) {
+    int lib_loglevel = atoi(lib_verbosity);
+    if (lib_loglevel > 0 && SysLogLevel <= lib_loglevel) {
+      SysLogLevel = lib_loglevel + 1;
+    }
+  }
+
   if (strrchr(argv[0],'/'))
     exec_name = strrchr(argv[0],'/')+1;
 
@@ -398,10 +429,15 @@ int main(int argc, char *argv[])
     case 'B': pes_buffers = atoi(optarg);
               PRINTF("Buffers: %d\n", pes_buffers);
               break;
+    case 10:  ffmpeg_threads = atoi(optarg);
+              PRINTF("FFmpeg thread count: %d\n", ffmpeg_threads);
+              break;
     case 'T': tty = optarg;
+#if 0
               if (access(tty, R_OK | W_OK) < 0) {
                 EXIT("Can't access terminal: %s\n", tty);
               }
+#endif
               PRINTF("Terminal: %s\n", tty);
               break;
     case 'n': scale_video = 0;
@@ -448,9 +484,11 @@ int main(int argc, char *argv[])
     case 'S': slave_mode = 1;
               PRINTF("Slave mode\n");
               break;
+#ifndef _WIN32
     case 'l': LogToSysLog = 1;
               openlog(exec_name, LOG_PID|LOG_CONS, LOG_USER);
               break;
+#endif
     case 'k': nokbd = 1;
               PRINTF("Keyboard input disabled\n");
               break;
@@ -477,6 +515,9 @@ int main(int argc, char *argv[])
     case 'R': reconnect = 1;
               PRINTF("Automatic reconnection enabled\n");
               break;
+    case 'c': waitvdr = 1;
+              PRINTF("Wait for VDR startup enabled\n");
+              break;
     case 't': ftcp = 1;
               PRINTF("Protocol: TCP\n");
               break;
@@ -499,12 +540,21 @@ int main(int argc, char *argv[])
 
   PRINTF("\n");
 
+#ifndef _WIN32
   if (tty) {
     /* claim new controlling terminal */
     stdin  = freopen(tty, "r", stdin);
     stdout = freopen(tty, "w", stdout);
+    if (!stdin || !stdout) {
+      EXIT("Can't access terminal: %s\n", tty);
+    }
     stderr = freopen(tty, "w", stderr);
+    if (!stderr) {
+      printf("Error reopening stderr\n");
+      exit(-1);
+    }
   }
+#endif
 
 #if 1
   /* backward compability */
@@ -516,42 +566,55 @@ int main(int argc, char *argv[])
 #endif
 
   /* If server address not given, try to find server automatically */
-  if (!mrl ||
+  while (!mrl ||
       !strcmp(mrl, MRL_ID ":") ||
       !strcmp(mrl, MRL_ID "+tcp:") ||
       !strcmp(mrl, MRL_ID "+udp:") ||
       !strcmp(mrl, MRL_ID "+rtp:") ||
       !strcmp(mrl, MRL_ID "+pipe:")) {
-    char address[1024] = "";
-    int port = -1;
+    vdr_server **svrs;
     PRINTF("VDR server not given, searching ...\n");
-    if (udp_discovery_find_server(&port, &address[0])) {
-      PRINTF("Found VDR server: host %s, port %d\n", address, port);
+    svrs = udp_discovery_find_servers(0);
+    if (svrs && svrs[0]) {
+      vdr_server *svr = *svrs;
+      PRINTF("Found VDR server(s):\n");
+      for (int s = 0; svrs[s]; s++)
+        PRINTF("\txvdr://%s:%d (%s)\n", svrs[s]->host, svrs[s]->port, svrs[s]->descr);
       if (mrl) {
         char *tmp = mrl;
         mrl = NULL;
-        if (asprintf(&mrl, "%s//%s:%d", tmp, address, port) < 0)
-          return -1;
+        if (asprintf(&mrl, "%s//%s:%d", tmp, svr->host, svr->port) < 0) {
+          free(tmp);
+          EXIT("asprintf failed");
+        }
         free(tmp);
-      } else
-        if (asprintf(&mrl, MRL_ID "://%s:%d", address, port) < 0)
-          return -1;
+      } else {
+        if (asprintf(&mrl, MRL_ID "://%s:%d", svr->host, svr->port) < 0)
+          EXIT("asprintf failed");
+      }
+      udp_discovery_free_servers(&svrs);
     } else {
+      if (waitvdr) {
+        sleep(1);
+        continue;
+      }
       PRINTF("---------------------------------------------------------------\n"
              "WARNING: MRL not given and server not found from local network.\n"
              "         Trying to connect to default port on local host.\n"
              "---------------------------------------------------------------\n");
+      free(mrl);
       mrl = strdup(MRL_ID "://127.0.0.1");
     }
   }
 
-  if (mrl &&
-      strncmp(mrl, MRL_ID ":", MRL_ID_LEN+1) &&
+  if (strncmp(mrl, MRL_ID ":", MRL_ID_LEN+1) &&
       strncmp(mrl, MRL_ID "+", MRL_ID_LEN+1)) {
     char *mrl2 = mrl;
     PRINTF("WARNING: MRL does not start with \'" MRL_ID ":\' (%s)\n", mrl);
-    if (asprintf(&mrl, MRL_ID "://%s", mrl) < 0)
-      return -1;
+    if (asprintf(&mrl, MRL_ID "://%s", mrl) < 0) {
+      free(mrl2);
+      EXIT("asprintf failed");
+    }
     free(mrl2);
   }
 
@@ -569,20 +632,24 @@ int main(int argc, char *argv[])
     }
   }
 
+#ifndef _WIN32
   if (daemon_mode) {
     PRINTF("Entering daemon mode\n\n");
     if (daemon(1, 0) == -1) {
       fprintf(stderr, "%s: %m\n", exec_name);
       LOGERR("daemon() failed");
-      return -2;
+      result = -2;
+      goto exit;
     }
   }
+#endif
 
   /* Create front-end */
   fe = (*fe_creator)();
   if (!fe) {
     fprintf(stderr, "Error initializing frontend\n");
-    return -3;
+    result = -3;
+    goto exit;
   }
 
   /* Initialize display */
@@ -591,17 +658,17 @@ int main(int argc, char *argv[])
                            video_port, scale_video,
                            aspect_controller, window_id)) {
     fprintf(stderr, "Error opening display\n");
-    fe->fe_free(fe);
-    return -4;
+    result = -4;
+    goto exit;
   }
 
   /* Initialize xine */
   if (!fe->xine_init(fe, audio_driver, audio_device, video_driver,
-                     pes_buffers, static_post_plugins, config_file)) {
+                     pes_buffers, ffmpeg_threads, static_post_plugins, config_file)) {
     fprintf(stderr, "Error initializing xine\n");
     list_xine_plugins(fe, SysLogLevel>2);
-    fe->fe_free(fe);
-    return -5;
+    result = -5;
+    goto exit;
   }
   if (power_off_cmd) {
     fe->shutdown_init(fe, power_off_cmd, inactivity_timer * 60);
@@ -611,11 +678,12 @@ int main(int argc, char *argv[])
     list_xine_plugins(fe, SysLogLevel>2);
 
   /* signal handlers */
-
+#ifndef _WIN32
   if (signal(SIGHUP,  SignalHandler) == SIG_IGN) signal(SIGHUP,  SIG_IGN);
   if (signal(SIGINT,  SignalHandler) == SIG_IGN) signal(SIGINT,  SIG_IGN);
   if (signal(SIGTERM, SignalHandler) == SIG_IGN) signal(SIGTERM, SIG_IGN);
   if (signal(SIGPIPE, SignalHandler) == SIG_IGN) signal(SIGPIPE, SIG_IGN);
+#endif
 
   do {
 
@@ -632,9 +700,14 @@ int main(int argc, char *argv[])
         PRINTF("Error opening %s\n", mrl);
         continue;
       }
+      if (waitvdr) {
+        PRINTF("Connection to server failed. Trying again after two seconds...\n");
+        sleep(2);
+        continue;
+      }
       fprintf(stderr, "Error opening %s\n", mrl);
-      fe->fe_free(fe);
-      return -6;
+      result = -6;
+      goto exit;
     }
 
     if (!fe->xine_play(fe)) {
@@ -643,21 +716,21 @@ int main(int argc, char *argv[])
         continue;
       }
       fprintf(stderr, "Error playing %s\n", argv[1]);
-      fe->fe_free(fe);
-      return -7;
+      result = -7;
+      goto exit;
     }
 
     if (firsttry) {
 
       /* Start LIRC forwarding */
-      lirc_start(fe, lirc_dev, repeat_emu);
+      lirc = lirc_start(fe, lirc_dev, repeat_emu, gui_hotkeys);
 
-      cec_start(fe, cec_hdmi_port, cec_dev_type);
+      cec = cec_start(fe, cec_hdmi_port, cec_dev_type);
 
       /* Start keyboard listener thread */
       if (!nokbd) {
         PRINTF("\n\nPress Esc to exit\n\n");
-        kbd_start(fe, slave_mode);
+        kbd = kbd_start(fe, slave_mode, gui_hotkeys);
       }
     }
 
@@ -672,12 +745,15 @@ int main(int argc, char *argv[])
 
     fe->xine_close(fe);
     firsttry = 0;
+    waitvdr = 0;
 
+#ifndef _WIN32
     /* HUP reconnects */
     if (last_signal == SIGHUP)
       last_signal = 0;
+#endif
 
-  } while (!last_signal && xine_finished != FE_XINE_EXIT && reconnect);
+  } while (!last_signal && xine_finished != FE_XINE_EXIT && (reconnect || (!reconnect && waitvdr)));
 
   /* Clean up */
 
@@ -685,13 +761,15 @@ int main(int argc, char *argv[])
 
   fe->send_event(fe, "QUIT");
 
-  /* stop input threads */
-  lirc_stop();
-  cec_stop();
-  if (!nokbd)
-    kbd_stop();
+ exit:
 
-  fe->fe_free(fe);
+  /* stop input threads */
+  lirc_stop(&lirc);
+  cec_stop(&cec);
+  kbd_stop(&kbd);
+
+  if (fe)
+    fe->fe_free(fe);
 
   free(static_post_plugins);
   free(mrl);
@@ -699,5 +777,5 @@ int main(int argc, char *argv[])
   free(video_driver);
   free(lirc_dev);
 
-  return xine_finished==FE_XINE_EXIT ? 0 : 1;
+  return result ? result : xine_finished==FE_XINE_EXIT ? 0 : 1;
 }
