@@ -56,7 +56,6 @@ END_PARAM_DESCR(audioch_param_descr)
 typedef struct audioch_post_plugin_s
 {
   post_plugin_t  post_plugin;
-  xine_post_in_t parameter_input;
 
   /* Config */
   int channel; /* 0 - left, 1 - right */
@@ -177,10 +176,14 @@ static xine_post_api_descr_t *audioch_get_param_descr(void)
   return &audioch_param_descr;
 }
 
+#if XINE_VERSION_CODE > 10209 || defined(PLUGIN_XINE_MODULE)
+static int audioch_set_parameters(xine_post_t *this_gen, const void *param_gen)
+#else
 static int audioch_set_parameters(xine_post_t *this_gen, void *param_gen)
+#endif
 {
+  const audioch_parameters_t *param = (const audioch_parameters_t *)param_gen;
   audioch_post_plugin_t *this = (audioch_post_plugin_t *)this_gen;
-  audioch_parameters_t *param = (audioch_parameters_t *)param_gen;
 
   this->channel = param->channel;
   return 1;
@@ -226,11 +229,18 @@ static post_plugin_t *audioch_open_plugin(post_class_t *class_gen,
   post_in_t             *input;
   post_out_t            *output;
   post_audio_port_t     *port;
-  xine_post_in_t        *input_param;
 
-  static xine_post_api_t post_api =
-    { audioch_set_parameters,  audioch_get_parameters, 
-      audioch_get_param_descr, audioch_get_help };
+  static const xine_post_api_t post_api = {
+    .set_parameters  = audioch_set_parameters,
+    .get_parameters  = audioch_get_parameters,
+    .get_param_descr = audioch_get_param_descr,
+    .get_help        = audioch_get_help,
+  };
+  static const xine_post_in_t input_param = {
+    .name = "parameters",
+    .type = XINE_POST_DATA_PARAMETERS,
+    .data = (void *)&post_api,
+  };
 
   if (!this || !audio_target || !audio_target[0] ) {
     free(this);
@@ -252,14 +262,10 @@ static post_plugin_t *audioch_open_plugin(post_class_t *class_gen,
   this->post_plugin.xine_post.audio_input[ 0 ] = &port->new_port;
   this->post_plugin.dispose = audioch_dispose;
 
-  input_param       = &this->parameter_input;
-  input_param->name = "parameters";
-  input_param->type = XINE_POST_DATA_PARAMETERS;
-  input_param->data = &post_api;
 #if XINE_VERSION_CODE >= 10102
-  xine_list_push_back(this->post_plugin.input, input_param);
+  xine_list_push_back(this->post_plugin.input, (void *)&input_param);
 #else
-  xine_list_append_content(this->post_plugin.input, input_param);
+  xine_list_append_content(this->post_plugin.input, (void *)&input_param);
 #endif
 
   this->channel = 0;
@@ -290,7 +296,11 @@ static void audioch_class_dispose(post_class_t *class_gen)
 }
 #endif
 
+#if XINE_VERSION_CODE > 10209 || defined(PLUGIN_XINE_MODULE)
+static void *audioch_init_plugin(xine_t *xine, const void *data)
+#else
 static void *audioch_init_plugin(xine_t *xine, void *data)
+#endif
 {
   post_class_t *class = calloc(1, sizeof(post_class_t));
   
@@ -318,6 +328,6 @@ plugin_info_t xine_plugin_info[] __attribute__((visibility("default"))) =
 {
   /* type, API, "name", version, special_info, init_function */
   { PLUGIN_POST, POST_PLUGIN_IFACE_VERSION, "audiochannel", XINE_VERSION_CODE, &audioch_info, &audioch_init_plugin },
-  { PLUGIN_NONE, 0, "", 0, NULL, NULL }
+  { PLUGIN_NONE, 0, NULL, 0, NULL, NULL }
 };
 #endif
